package nckd.base.helper;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.fileservice.FileServiceFactory;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.AttachmentDto;
import kd.bos.servicehelper.AttachmentServiceHelper;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLContextBuilder;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.entity.AbstractHttpEntity;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import javax.net.ssl.SSLContext;

import java.io.*;
import java.security.cert.CertificateException;
import java.util.List;
import java.util.Map;


public class FileSECUtils {
    private static final Log logger = LogFactory.getLog(FileSECUtils.class);

    public static InputStream getFileTest() {

        String entryEntityName = "nckd_filetest";

        DynamicObject billByn = BusinessDataServiceHelper.loadSingle(entryEntityName,
                new QFilter[]{new QFilter("billno", QCP.equals, "test0001")});

        List<Map<String, Object>> sourceFileList = AttachmentServiceHelper.getAttachments(entryEntityName,billByn.getPkValue(), "attachmentpanel");

        if(sourceFileList != null && sourceFileList.size()>0)
        {
            for(Map<String,Object> fileMap : sourceFileList){
                AttachmentDto attachmentDto = AttachmentServiceHelper.getAttachmentInfoByAttPk(fileMap.get("attPkId"));
                String fileUrl= attachmentDto.getResourcePath();
                String fileName = attachmentDto.getFilename();

                System.out.println("fileUrl:" + fileUrl+ ",fileName:"+fileName);
                ByteArrayOutputStream out = new ByteArrayOutputStream();
                FileServiceFactory.getAttachmentFileService().download(fileUrl, out, null);
                InputStream in = new ByteArrayInputStream(out.toByteArray());
                return in;
            }
        }

        return null;
    }

    /**
     * 检查文件是否加密的REST接口调用方法
     * 该方法会从配置中获取检查URL，构造请求并发送文件流进行检测。
     * 返回结果说明：
     *   1: 文件是秘文（加密状态）
     *   0: 文件是明文（未加密状态）
     *  -1: 发生错误或未找到配置信息
     */
    public static int checkFileIsEncryptionRest(InputStream inputStream){

        Map<String, String> mapentity = CommonHelperUtils.getCommonParams("SEC");
        if(mapentity == null){
            return -1;
        }
        String url = mapentity.get("url");

        try {
            CloseableHttpClient httpclient = null;
            HttpPost post = new HttpPost(url);
            post.addHeader("method~name", "checkFileIsEncryptionRest");//文件加密

            //String filePath = "C:\\Users\\Administrator\\Desktop\\test.txt";
            //InputStream inputStream = new FileInputStream(filePath);

            AbstractHttpEntity entity = new InputStreamEntity(inputStream);
            post.setEntity(entity);

            httpclient = buildSSLCloseableHttpClient();
            CloseableHttpResponse response = httpclient.execute(post);
            String result = response.getFirstHeader("data~returnFlag").getValue();

            // 获取响应状态码
            if("1".equals(result)){//1表示秘文
                System.out.println("是秘文");
                return 1;
            }else if("0".equals(result)){//0表示明文
                System.out.println("是明文");
                return 0;
            }else{
                System.out.println(result);
                return -1;
            }

        } catch (IOException e) {
            e.printStackTrace();
            logger.info("SEC附件 ERR content1: " + e.getMessage());
        } catch (Exception e1){
            e1.printStackTrace();
            logger.info("SEC附件 ERR content2: " + e1.getMessage());
        }
        return -1;
    }

    public static InputStream encodeFileForSEC(InputStream inputStream){

        Map<String, String> mapentity = CommonHelperUtils.getCommonParams("SEC");
        if(mapentity == null){
            return null;
        }
        String url = mapentity.get("url");

        try {

            String filePath = "C:\\Users\\turbo_i49t4d3\\Downloads\\副本经理人周会数据.xlsx";
            File file = new File(filePath);

            CloseableHttpClient httpclient = null;
            HttpPost post = new HttpPost(url);
            post.addHeader("method~name", "fileEncryptionRest"); //文件加密
            post.addHeader("data~fileOffset", "0");
            post.addHeader("data~counSize", file.length()+"");
            post.addHeader("data~secLevel", "10");

            //String filePath = "C:\\Users\\Administrator\\Desktop\\test.txt";
            InputStream inputStream1 = new FileInputStream(filePath);

            AbstractHttpEntity entity = new InputStreamEntity(inputStream1);
            post.setEntity(entity);

            httpclient = buildSSLCloseableHttpClient();
            CloseableHttpResponse response = httpclient.execute(post);
            String result = response.getFirstHeader("data~returnFlag").getValue();

            // 获取响应状态码
            if("0".equals(result)){//1表示秘文
                InputStream ins = response.getEntity().getContent();
                logger.info("SEC附件加密成功: " + result);
                return ins;
            }else{
                logger.debug("SEC附件加密失败: " + result);
                return null;
            }

        } catch (IOException e) {
            e.printStackTrace();
            logger.info("SEC附件加密 ERR content1: " + e.getMessage());
        } catch (Exception e1){
            e1.printStackTrace();
            logger.info("SEC附件加密 ERR content2: " + e1.getMessage());
        }
        return null;
    }

    public static InputStream decodeFileForSEC(InputStream inputStream){
        Map<String, String> mapentity = CommonHelperUtils.getCommonParams("SEC");
        if(mapentity == null){
            return null;
        }
        String url = mapentity.get("url");

        try {

            String filePath = "C:\\Users\\turbo_i49t4d3\\Downloads\\PMS项目预算编制维度-01.xlsx";
            File file = new File(filePath);

            CloseableHttpClient httpclient = null;
            HttpPost post = new HttpPost(url);
            post.addHeader("method~name", "fileDecryptionRest"); //文件解密
            post.addHeader("data~fileOffset", "0");
            post.addHeader("data~counSize", file.length()+"");

            InputStream inputStream1 = new FileInputStream(filePath);

            AbstractHttpEntity entity = new InputStreamEntity(inputStream1);
            post.setEntity(entity);

            httpclient = buildSSLCloseableHttpClient();
            CloseableHttpResponse response = httpclient.execute(post);
            String result = response.getFirstHeader("data~returnFlag").getValue();

            // 获取响应状态码
            if("0".equals(result)){//1表示秘文
                InputStream ins = response.getEntity().getContent();
                logger.info("SEC附件解密成功: " + result);
                return ins;
            }else{
                logger.info("SEC附件解密失败: " + result);
                return null;
            }

        } catch (IOException e) {
            e.printStackTrace();
            logger.info("SEC附件解密 ERR content1: " + e.getMessage());
        } catch (Exception e1){
            e1.printStackTrace();
            logger.info("SEC附件解密 ERR content2: " + e1.getMessage());
        }
        return null;
    }

    private static CloseableHttpClient buildSSLCloseableHttpClient() throws Exception {
        SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
            @Override
            public boolean isTrusted(java.security.cert.X509Certificate[] x509Certificates, String s) throws CertificateException {
                return true;
            }
        }).build();
        // ALLOW_ALL_HOSTNAME_VERIFIER:这个主机名验证器基本上是关闭主机名验证的,实现的是一个空操作，并且不会抛出javax.net.ssl.SSLException异常。
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, new String[] { "TLSv1" }, null,
                SSLConnectionSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
        return HttpClients.custom().setSSLSocketFactory(sslsf).build();
    }

}
